/* test07.c - play sound, via interrupt handler */

#include <mikeos.h>

/*
	used MikeOS API :-

	mikeos_speaker_tone() (call from assembler)
	mikeos_speaker_off()
	mikeos_pause()

	mikeos_print_digit()
	mikeos_print_4hex()
	mikeos_print_space()
	mikeos_print_newline()
*/

static	void	interrupt_handler(void)
{
	/*
	  When interrupt is raised, current context --- flag register
	  and instruction pointer (where interrupt handler is exited) ---
	  is saved onto stack. After saving context, CPU runs
	  interrupt handler.

		Note: Other registers (ex. AX, BX, CX, ...) is *not*
		saved by CPU. So, if interrupt handler handles any register,
		interrupt handler must save before use.

	  libmikeos uses 32bit registers to handle long/unsigned long
	  value, interrupt handler have to start with pushd instruction, and
	  exits with popd and iret instruction.
	*/

#asm
	! In bcc's inline assembler, comment starts with '!'.
	! You do not have to save/restore AX/BX/CX/DX/ES registers,
	! but others must be preserved.

	! save current context
	pushad

	! interrupt context come here...
	! this sample use os_speaker_tone,
	! because this API is I/O port operation only - good for test.
	! caller must set ax register before calling.
	mov	bx, #0x801b
	call bx

	! return to original context
	popad
	iret
#endasm	

	/* do not add "return" here because bcc adds garbage */
}

int	MikeMain(void *arg)
{
	/* mikeos_get_int_handler() accepts both pointer to
	   static and stack region */
	static	unsigned int	segment0, offset0;
	unsigned int		segment1, offset1;

#define	IntVec	0xf1

	/* Use "real" PC, emulators are not recommended */

	/* save current interrupt vector */
	mikeos_get_int_handler(IntVec, &segment0, &offset0);

	/* set new interrupt vector */
	mikeos_modify_int_handler(IntVec, interrupt_handler);

	/* get new interrupt vector */
	mikeos_get_int_handler(IntVec, &segment1, &offset1);

	/* show interrupt vector before/after */
	mikeos_print_4hex(segment0);
	mikeos_print_space();
	mikeos_print_4hex(offset0);
	mikeos_print_newline();
	mikeos_print_4hex(segment1);
	mikeos_print_space();
	mikeos_print_4hex(offset1);
	mikeos_print_newline();

	/* play 880Hz sound via interrupt handler */
	mikeos_print_digit(1);
	mikeos_print_space();

#asm
	mov	ax, #1356		! 1193182Hz / 880Hz = 1355.827273
	int	#IntVec			! software interrupt
#endasm

	mikeos_pause(1000000UL);	/* 1sec wait, "unsigned long" */
	mikeos_speaker_off();

	mikeos_print_digit(0);
	mikeos_print_newline();

	/*
	  remember to restore original interrupt vector,
	  but mikeos_modify_int_handler() cannot restore original segment.
	  So this sample restores offset only.

	  If you handle hardware-interrupt, 
	  disable it before remove interrupt handler!
	*/
	mikeos_modify_int_handler(IntVec, offset0);

	return 0;
}
